/* Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 *
 * @defgroup ble_sdk_app_template_main main.c
 * @{
 * @ingroup ble_sdk_app_template
 * @brief Template project main file.
 *
 * This file contains a template for creating a new application. It has the code necessary to wakeup
 * from button, advertise, get a connection restart advertising on disconnect and if no new
 * connection created go back to system-off mode.
 * It can easily be used as a starting point for creating a new application, the comments identified
 * with 'YOUR_JOB' indicates where and how you can customize.
 */

 
 
#include <stdbool.h>
#include <stdio.h>
#include "nrf_adc.h"
#include "app_uart.h"
#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "nrf51_bitfields.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "boards.h"
#include "softdevice_handler.h"
#include "app_timer.h"
#include "device_manager.h"
#include "pstorage.h"
#include "app_trace.h"
#include "bsp.h"
#include "bsp_btn_ble.h"
#include "nrf_delay.h"
#include "nrf_gpiote.h"

#define APP_CFG_NON_CONN_ADV_TIMEOUT    0                                 /**< Time for which the device must be advertising in non-connectable mode (in seconds). 0 disables timeout. */
#define NON_CONNECTABLE_ADV_INTERVAL    MSEC_TO_UNITS(500, UNIT_0_625_MS) /**< The advertising interval for non-connectable advertisement (100 ms). This value can vary between 100ms to 10.24s). */

#define APP_BEACON_INFO_LENGTH          0x17                              /**< Total length of information advertised by the Beacon. */
#define APP_ADV_DATA_LENGTH             0x15                              /**< Length of manufacturer specific data in the advertisement. */
#define APP_DEVICE_TYPE                 0x02                              /**< 0x02 refers to Beacon. */
#define APP_MEASURED_RSSI               0xC3     
#define APP_COMPANY_IDENTIFIER          0xFFFF 

#define IS_SRVC_CHANGED_CHARACT_PRESENT  1                                          /**< Include or not the service_changed characteristic. if not enabled, the server's database cannot be changed for the lifetime of the device*/

#define DEVICE_NAME                      "SiBeacon"                               /**< Name of device. Will be included in the advertising data. */
#define MANUFACTURER_NAME                "SadeArGe"                      /**< Manufacturer. Will be passed to Device Information Service. */
#define APP_ADV_INTERVAL                 500                                        /**< The advertising interval (in units of 0.625 ms. This value corresponds to 25 ms). */
#define APP_ADV_TIMEOUT_IN_SECONDS       0                                        /**< The advertising timeout in units of seconds. */

#define APP_TIMER_PRESCALER              0                                          /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_MAX_TIMERS             (6+BSP_APP_TIMERS_NUMBER)                  /**< Maximum number of simultaneously created timers. */
#define APP_TIMER_OP_QUEUE_SIZE          4                                          /**< Size of timer operation queues. */

#define MIN_CONN_INTERVAL                MSEC_TO_UNITS(100, UNIT_1_25_MS)           /**< Minimum acceptable connection interval (0.1 seconds). */
#define MAX_CONN_INTERVAL                MSEC_TO_UNITS(200, UNIT_1_25_MS)           /**< Maximum acceptable connection interval (0.2 second). */
#define SLAVE_LATENCY                    0                                          /**< Slave latency. */
#define CONN_SUP_TIMEOUT                 MSEC_TO_UNITS(4000, UNIT_10_MS)            /**< Connection supervisory timeout (4 seconds). */

#define FIRST_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(5000, APP_TIMER_PRESCALER) /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY    APP_TIMER_TICKS(30000, APP_TIMER_PRESCALER)/**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT     3                                          /**< Number of attempts before giving up the connection parameter negotiation. */

static ble_gap_adv_params_t m_adv_params;                                 /**< Parameters to be passed to the stack when starting advertising. */
#define SEC_PARAM_BOND                   1                                          /**< Perform bonding. */
#define SEC_PARAM_MITM                   0                                          /**< Man In The Middle protection not required. */
#define SEC_PARAM_IO_CAPABILITIES        BLE_GAP_IO_CAPS_NONE                       /**< No I/O capabilities. */
#define SEC_PARAM_OOB                    0                                          /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE           7                                          /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE           16                                         /**< Maximum encryption key size. */

#define DEAD_BEEF                        0xDEADBEEF                                 /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */



#define LED1	5
#define	LED2	4

#define IDLE_TIME 300000//10000 

//#define UICR_ADDRESS                    0x10001080                        /**< Address of the UICR register used by this example. The major and minor versions to be encoded into the advertising data will be picked up from this location. */



static dm_application_instance_t        m_app_handle;                               /**< Application identifier allocated by device manager */

static uint16_t                          m_conn_handle = BLE_CONN_HANDLE_INVALID;   /**< Handle of the current connection. */
static app_timer_id_t										IDLE_TIMER;
//static app_timer_id_t										ADVERTISING_STOP_TIMER;
//static app_timer_id_t										TWO_MINUTES_SUPERVISION_TIMER;
//static app_timer_id_t										ACTIVITY_ONE_SECOND_TIMER;
//static app_timer_id_t										IDLE_TIMER_QUEUE;
//static app_timer_id_t										TWO_MINUTES_SUPERVISION_TIMER_QUEUE;
//static app_timer_id_t										m_app_timer_id_7;
///////////////
uint8_t get_battery_pct(void);
void build_scan_response_data(void);
static void advertising_init(void);
volatile uint32_t adc_sample;
volatile uint8_t bat_pct;
volatile uint32_t working_time_in_seconds=0;
uint32_t idle_signal_counter=0;
uint32_t detached_counter=0;
bool ACTIVITY_ONE_SECOND_TIMER_IS_ACTIVE=false;
bool IS_ADVERTISING=false;
uint8_t starting_adv_data[13] = {0};
uint8_t sr_data[24] = {0};
uint8_t adv_data[30]={0};

uint32_t idle_counter=6;
	#define ADC_VBAT_CONFIG { NRF_ADC_CONFIG_RES_10BIT,               \
                                 NRF_ADC_CONFIG_SCALING_SUPPLY_ONE_THIRD, \
                                 NRF_ADC_CONFIG_REF_VBG }
/* YOUR_JOB: Declare all services structure your application is using
static ble_xx_service_t                     m_xxs;
static ble_yy_service_t                     m_yys;
*/

// YOUR_JOB: Use UUIDs for service(s) used in your application.
//static ble_uuid_t m_adv_uuids[] = {{BLE_UUID_DEVICE_INFORMATION_SERVICE, BLE_UUID_TYPE_BLE}}; /**< Universally unique service identifiers. */

                                   
/**@brief Callback function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num   Line number of the failing ASSERT call.
 * @param[in] file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}

static void advertising_init(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
    uint8_t       flags = BLE_GAP_ADV_FLAG_BR_EDR_NOT_SUPPORTED;

    ble_advdata_manuf_data_t manuf_specific_data;

    manuf_specific_data.company_identifier = APP_COMPANY_IDENTIFIER;


  //  manuf_specific_data.data.p_data = (uint8_t *) m_beacon_info;
    manuf_specific_data.data.size   = APP_BEACON_INFO_LENGTH;

    // Build and set advertising data.
    memset(&advdata, 0, sizeof(advdata));

   // advdata.name_type             = BLE_ADVDATA_NO_NAME;
    advdata.flags                 = flags;
    advdata.p_manuf_specific_data = &manuf_specific_data;

    err_code = ble_advdata_set(&advdata, NULL); // i m not sure to use about this func beacouse in api it calls sd_ble_gap_adv_data_set()  like below
    APP_ERROR_CHECK(err_code);

    // Initialize advertising parameters (used when starting advertising).
    memset(&m_adv_params, 0, sizeof(m_adv_params));
	//ble_gap_adv_ch_mask_t channels;
	//channels.ch_39_off=1;
	//channels.ch_38_off=1;
	//channels.ch_37_off=1;
	//	m_adv_params.channel_mask=channels;		
	//m_adv_params.channel_mask=channels;
    m_adv_params.type        = BLE_GAP_ADV_TYPE_ADV_SCAN_IND;// this is for scan request packet
    m_adv_params.p_peer_addr = NULL;                             // Undirected advertisement.
    m_adv_params.fp          = BLE_GAP_ADV_FP_ANY;
    m_adv_params.interval    = NON_CONNECTABLE_ADV_INTERVAL;
    m_adv_params.timeout     = APP_CFG_NON_CONN_ADV_TIMEOUT; //set to zero and no timeout up to  updating scan response packet  
		
	err_code = sd_ble_gap_adv_data_set(adv_data, 31, sr_data, 24);
    APP_ERROR_CHECK(err_code);
}
static void advertising_start(void)
{
    uint32_t err_code;

    err_code = sd_ble_gap_adv_start(&m_adv_params);
    APP_ERROR_CHECK(err_code);


}
void IDLE_TIMER_handler(void* p_context)
{
	idle_counter++;
	if(idle_counter >= 6)
	{	
		idle_counter=0;
		idle_signal_counter+=1;

		uint32_t err_code;
		err_code=sd_ble_gap_adv_stop();
		APP_ERROR_CHECK(err_code);
		build_scan_response_data();
		advertising_init();
		advertising_start();
	}	
}

/**@brief Function for the Timer initialization.
 *
 * @details Initializes the timer module. This creates and starts application timers.
 */
static void timers_init(void)
{

    // Initialize timer module.
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, false);

    // Create timers.

    /* YOUR_JOB: Create any timers to be used by the application.
                 Below is an example of how to create a timer.
                 For every new timer needed, increase the value of the macro APP_TIMER_MAX_TIMERS by
                 one.
	*/
    uint32_t err_code;
		err_code = app_timer_create(&IDLE_TIMER, APP_TIMER_MODE_REPEATED, IDLE_TIMER_handler);
	//	err_code = app_timer_create(&ADVERTISING_STOP_TIMER, APP_TIMER_MODE_SINGLE_SHOT, ADVERTISING_STOP_TIMER_handler);
	//	err_code = app_timer_create(&TWO_MINUTES_SUPERVISION_TIMER, APP_TIMER_MODE_SINGLE_SHOT, TWO_MINUTES_SUPERVISION_TIMER_handler);
   // err_code = app_timer_create(&ACTIVITY_ONE_SECOND_TIMER, APP_TIMER_MODE_REPEATED, ACTIVITY_ONE_SECOND_TIMER_handler);
	//	err_code = app_timer_create(&IDLE_TIMER_QUEUE, APP_TIMER_MODE_SINGLE_SHOT, IDLE_TIMER_QUEUE_handler);
   // err_code = app_timer_create(&TWO_MINUTES_SUPERVISION_TIMER_QUEUE, APP_TIMER_MODE_SINGLE_SHOT, TWO_MINUTES_SUPERVISION_TIMER_QUEUE_handler);
		//err_code = app_timer_create(&m_app_timer_id_7, APP_TIMER_MODE_REPEATED, m_app_timer_id_7_handler);
    APP_ERROR_CHECK(err_code); 
}


/**@brief Function for the GAP initialization.
 *
 * @details This function sets up all the necessary GAP (Generic Access Profile) parameters of the
 *          device including the device name, appearance, and the preferred connection parameters.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *)DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    /* YOUR_JOB: Use an appearance value matching the application's use case.
    err_code = sd_ble_gap_appearance_set(BLE_APPEARANCE_);
    APP_ERROR_CHECK(err_code); */

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}



/**@brief Function for handling the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module which
 *          are passed to the application.
 *          @note All this function does is to disconnect. This could have been done by simply
 *                setting the disconnect_on_fail config parameter, but instead we use the event
 *                handler mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
    }
}


/**@brief Function for handling a Connection Parameters error.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for starting timers.
*/
static void application_timers_start(void)
{
    /* YOUR_JOB: Start your timers. below is an example of how to start a timer.*/
    uint32_t err_code;
			err_code = app_timer_start(IDLE_TIMER, APP_TIMER_TICKS(IDLE_TIME, APP_TIMER_PRESCALER), NULL);
	//	err_code = app_timer_start(ADVERTISING_STOP_TIMER, APP_TIMER_TICKS(10000, APP_TIMER_PRESCALER), NULL);
	//	err_code = app_timer_start(TWO_MINUTES_SUPERVISION_TIMER, APP_TIMER_TICKS(10000, APP_TIMER_PRESCALER), NULL);
	//	err_code = app_timer_start(ACTIVITY_ONE_SECOND_TIMER, APP_TIMER_TICKS(1000, APP_TIMER_PRESCALER), NULL);
	//	err_code = app_timer_start(m_app_timer_id_5, APP_TIMER_TICKS(10000, APP_TIMER_PRESCALER), NULL);
	//	err_code = app_timer_start(TWO_MINUTES_SUPERVISION_TIMER_QUEUE, APP_TIMER_TICKS(10000, APP_TIMER_PRESCALER), NULL);
		//err_code = app_timer_start(m_app_timer_id_7, APP_TIMER_TICKS(10000, APP_TIMER_PRESCALER), NULL);


    APP_ERROR_CHECK(err_code); 
   
}





/**@brief Function for handling the Application's BLE Stack events.
 *
 * @param[in] p_ble_evt  Bluetooth stack event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    uint32_t err_code;

    switch (p_ble_evt->header.evt_id)
            {
        case BLE_GAP_EVT_CONNECTED:
            //err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            //APP_ERROR_CHECK(err_code);
           // m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
			m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
			err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
            APP_ERROR_CHECK(err_code);
				
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            break;

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for dispatching a BLE stack event to all modules with a BLE stack event handler.
 *
 * @details This function is called from the BLE Stack event interrupt handler after a BLE stack
 *          event has been received.
 *
 * @param[in] p_ble_evt  Bluetooth stack event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
    dm_ble_evt_handler(p_ble_evt);
    ble_conn_params_on_ble_evt(p_ble_evt);
    bsp_btn_ble_on_ble_evt(p_ble_evt);
    on_ble_evt(p_ble_evt);
    ble_advertising_on_ble_evt(p_ble_evt);
    /*YOUR_JOB add calls to _on_ble_evt functions from each service your application is using
    ble_xxs_on_ble_evt(&m_xxs, p_ble_evt);
    ble_yys_on_ble_evt(&m_yys, p_ble_evt);
    */
}


/**@brief Function for dispatching a system event to interested modules.
 *
 * @details This function is called from the System event interrupt handler after a system
 *          event has been received.
 *
 * @param[in] sys_evt  System stack event.
 */
static void sys_evt_dispatch(uint32_t sys_evt)
{
    pstorage_sys_event_handler(sys_evt);
    ble_advertising_on_sys_evt(sys_evt);
}


/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    uint32_t err_code;

    // Initialize the SoftDevice handler module.
    SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_XTAL_20_PPM, NULL);

#if defined(S110) || defined(S130)
    // Enable BLE stack.
    ble_enable_params_t ble_enable_params;
    memset(&ble_enable_params, 0, sizeof(ble_enable_params));
#ifdef S130
    ble_enable_params.gatts_enable_params.attr_tab_size   = BLE_GATTS_ATTR_TAB_SIZE_DEFAULT;
#endif
    ble_enable_params.gatts_enable_params.service_changed = IS_SRVC_CHANGED_CHARACT_PRESENT;
    err_code = sd_ble_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);
#endif

    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);

    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_sys_evt_handler_set(sys_evt_dispatch);
    APP_ERROR_CHECK(err_code);
}



/**@brief Function for handling the Device Manager events.
 *
 * @param[in] p_evt  Data associated to the device manager event.
 */
static uint32_t device_manager_evt_handler(dm_handle_t const * p_handle,
                                           dm_event_t const  * p_event,
                                           ret_code_t        event_result)
{
    APP_ERROR_CHECK(event_result);

#ifdef BLE_DFU_APP_SUPPORT
    if (p_event->event_id == DM_EVT_LINK_SECURED)
    {
        app_context_load(p_handle);
    }
#endif // BLE_DFU_APP_SUPPORT

    return NRF_SUCCESS;
}


/**@brief Function for the Device Manager initialization.
 *
 * @param[in] erase_bonds  Indicates whether bonding information should be cleared from
 *                         persistent storage during initialization of the Device Manager.
 */
static void device_manager_init(bool erase_bonds)
{
    uint32_t               err_code;
    dm_init_param_t        init_param = {.clear_persistent_data = erase_bonds};
    dm_application_param_t register_param;

    // Initialize persistent storage module.
    err_code = pstorage_init();
    APP_ERROR_CHECK(err_code);

    err_code = dm_init(&init_param);
    APP_ERROR_CHECK(err_code);

    memset(&register_param.sec_param, 0, sizeof(ble_gap_sec_params_t));

    register_param.sec_param.bond         = SEC_PARAM_BOND;
    register_param.sec_param.mitm         = SEC_PARAM_MITM;
    register_param.sec_param.io_caps      = SEC_PARAM_IO_CAPABILITIES;
    register_param.sec_param.oob          = SEC_PARAM_OOB;
    register_param.sec_param.min_key_size = SEC_PARAM_MIN_KEY_SIZE;
    register_param.sec_param.max_key_size = SEC_PARAM_MAX_KEY_SIZE;
    register_param.evt_handler            = device_manager_evt_handler;
    register_param.service_type           = DM_PROTOCOL_CNTXT_GATT_SRVR_ID;

    err_code = dm_register(&m_app_handle, &register_param);
    APP_ERROR_CHECK(err_code);
}




void gpio_init(void)
{
	nrf_gpio_pin_dir_set(LED1,NRF_GPIO_PIN_DIR_OUTPUT);
	
	//nrf_gpio_pin_dir_set(30,NRF_GPIO_PIN_DIR_OUTPUT);
	//nrf_gpio_pin_dir_set(7,NRF_GPIO_PIN_DIR_OUTPUT);
	
	nrf_gpio_pin_set(LED1);
	nrf_delay_ms(1000);
	nrf_gpio_pin_clear(LED1);
}		






	



uint8_t get_battery_pct(void)
{
	/*adc result is 10 bit range 0-1023 input vdd/3 upper reference 1,2V lower reference GND full battery assume 3,1V empty battery 2,3V
	1,2V--->1023								||1,2V--->1023
	3,1/3 ---X ==>x=880				||2,3/3--->Y ==> Y=596
	to make adc_sample zero starting adc_sample=adc_sample-596
	scaling to percentage battery 
	bat_pct=adc_sample*100/(880-596)
	*/
		adc_sample= nrf_adc_convert_single	(	NRF_ADC_CONFIG_INPUT_DISABLED	)	;
		nrf_adc_stop();
		if (adc_sample >=880)
		{
			adc_sample=880;																				
		}
		else if (adc_sample<880 && adc_sample >596)						
		{
			adc_sample=adc_sample;														
		}																								
		else 																						
		{
			adc_sample=596;															
		}
		
		adc_sample=adc_sample-596;

		adc_sample=(adc_sample*100);
		adc_sample=adc_sample/284;
		bat_pct=adc_sample;
		return bat_pct;
		
}
/**
 * @brief ADC initialization.
 */
void adc_config(void)
{
  //  const nrf_adc_config_t nrf_adc_config = NRF_ADC_CONFIG_DEFAULT;
	const nrf_adc_config_t nrf_adc_config = ADC_VBAT_CONFIG;
	
    // Initialize and configure ADC
    nrf_adc_configure( (nrf_adc_config_t *)&nrf_adc_config);

}
void build_adv_data(void)
{
	uint64_t deviceAddr = ((uint64_t)NRF_FICR->DEVICEADDR[1] << 32) | ((uint64_t)NRF_FICR->DEVICEADDR[0]);
	
	adv_data[0]=0x02;
	adv_data[1]=0x01;
	adv_data[2]=0x04;
	
	adv_data[3]=0x1A;//length 26 byte
	adv_data[4]=0xFF;// man. specific data
	adv_data[5]=0xFF;//comp id
	adv_data[6]=0xFF;//comp id

	adv_data[7]=0x02;//represent beacon
	adv_data[8]=0x15;//21 byte remain part
	
	adv_data[9]=0xE2;//start of 128 bit uuid
	adv_data[10]=0xC5;
	adv_data[11]=0x6D;
	adv_data[12]=0xB5;
	adv_data[13]=0xDF;
	adv_data[14]=0xFB;
	adv_data[15]=0x48;
	adv_data[16]=0xd2;
	adv_data[17]=0xb0;
	adv_data[18]=0x60;
	adv_data[19]=0xD0;
	adv_data[20]=0xf5;
	adv_data[21]=0xA7;
	adv_data[22]=0x10;
	adv_data[23]=0x96;
	adv_data[24]=0xE0;// end of 128bit uuid
	
    adv_data[25] = deviceAddr>>24;// mac address xx:xx:XX:xx:xx:xx major 
    adv_data[26] = deviceAddr>>16;//mac address  xx:xx:xx:XX:xx:xx major

    adv_data[27] = deviceAddr>>8;//mac address   xx:xx:xx:xx:XX:xx minor
    adv_data[28] = deviceAddr;//                 xx:xx:xx:xx:xx:XX minor
 
	//measured tx power -58dbm
	adv_data[29]=0xC5;
	
	
}
void build_scan_response_data(void)
{
	    // -------------------------------
    // build custom scan response data
    // -------------------------------

    // get Bluetooth MAC address of this device (NOTE: this is little-endian!)

    sr_data[0] = 0x09;  
    sr_data[1] = 0x09;  
    sr_data[2] = 0x53;  
    sr_data[3] = 0x69;  
    sr_data[4] = 0x42;  
	sr_data[5] = 0x65;
	sr_data[6] = 0x61;
	sr_data[7] = 0x63;
	sr_data[8] = 0x6F;
	sr_data[9] = 0x6E;
	
	sr_data[10] = 0x02;
	sr_data[11] = 0x0A;
	sr_data[12] = 0x04;
	
	sr_data[13] = 0x0A;
	sr_data[14] = 0x16; //service data
	sr_data[15] = 0x0D; //16bit uuid
	sr_data[16] = 0xD0;// 16bit uuid
	sr_data[17] = idle_signal_counter>>24;  //counter value 
	sr_data[18] = idle_signal_counter>>16;  //counter value
	sr_data[19] = idle_signal_counter>>8;   //counter value
	sr_data[20] = idle_signal_counter;      //counter value
	sr_data[21]=0x00;
	sr_data[22]=0x01;
	sr_data[23]=get_battery_pct();

	
}


void init_module(void)
{	int *temp; // used to enter timer handler function 
	gpio_init();
	adc_config();
    uint32_t err_code;
    bool erase_bonds;
    timers_init();
   // buttons_leds_init(&erase_bonds);
    ble_stack_init();
    device_manager_init(erase_bonds);
    gap_params_init();
	build_adv_data();
	build_scan_response_data();
	advertising_init();
    conn_params_init();
	err_code=sd_ble_gap_tx_power_set(4);//tx_power Radio transmit power in dBm (accepted values are -40, -30, -20, -16, -12, -8, -4, 0, and 4 dBm).
	APP_ERROR_CHECK(err_code);
		advertising_start(); // start advertising for first time
		//nrf_delay_ms(1000);	
	//	adv_data[11]=get_battery_pct();
    // Start execution.
   
	 application_timers_start();

		
		
    APP_ERROR_CHECK(err_code);
		
		IDLE_TIMER_handler(temp);  // goto timer handler and stop advertising and update scan response data and start again advertising
}



/**@brief Function for the Power manager.
 */
static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}
/**@brief Function for application main entry.
 */
int main(void)	
{		init_module();
		
		 // Enter main loop.	
	

    for (;;)
    {
        power_manage();
    }
}


void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
   // NVIC_SystemReset();
	//nrf_gpio_pin_toggle(LED1);
}



